#
# client.py - routines for snake client programs
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import snake.tree, snake.uri
import xmlrpclib
from snake.constants import *

# Minimum snake XMLRPC API version
minimum_version = 1

def connect(server, port=2903):
    '''Connect to the given server url. Returns a ServerProxy object.'''
    server_url = "http://%s:%s" % (server, port)
    return xmlrpclib.ServerProxy(server_url)

def check_server(server, port=2903):
    '''Check the server given by the URL.'''
    server_url = "http://%s:%s" % (server, port)
    try:
        server = xmlrpclib.ServerProxy(server_url)
        methods = server.system.listMethods()
    except:
        return False

    if 'snake.version' not in methods:
        return False
    if server.snake.version() < minimum_version:
        return False

    return True

def check_tree(uri,arch=None):
    '''Check a tree to see if it's usable. Tries a little harder than
    snake.uri.check to return a detailed response.
    Returns 0 on success, 1 if the tree is gone, 2 if the tree has bad
    metadata (.treeinfo etc), 3 if the tree has no boot images.'''
    try:
        # If this block fails, status is 'gone'
        # You can't get a valid fd by uriopening a FTP directory, so this
        # test doesn't work for FTP uris. Skip 'em.
        u = snake.uri.parse(uri)
        if u.scheme not in ['ftp','nfs']:
            r = CHECK_TREE_GONE
            snake.uri.uriopen(uri)
        # If this block fails, status is 'bad metadata'
        r = CHECK_TREE_BROKEN 
        t = snake.uri.uri_to_tree(uri)

        # FIXME - the tree arch doesn't always match the images provided (e.g. ppc)
        if not arch:
            arch = t.arch

        # If this block fails, status is 'no images'
        r = CHECK_TREE_NOIMAGES
        if t.images.has_key(arch) and len(t.images[arch]) > 0:
            # we're OK!
            r = CHECK_TREE_SUCCESS
    except snake.uri.URIError, e:
        pass
    return r

def treedict(server,**args):
    '''Return a dict of trees (using tree.id as the keys) that match the args
    given. Args will be passed to the server tree.list method.'''
    td = [(t['id'],snake.tree.Tree(dict=t)) for t in server.tree.list(args)]
    return dict(td)

def userconfirm(msg="Is this ok"):
    '''Prompt the user to entry y for yes, or N for no
       Returns True for yes, and False for no'''

    while True:
        choice = raw_input("%s [y/N]: " % msg)
        choice = choice.lower()
        if len(choice) == 0 or choice[0] in ['y', 'n']:
            break

    if len(choice) == 0 or choice[0] != 'y':
        return False
    else:
        return True

